@@ -23,11 +23,11 @@
 static ID s_replace;
 #endif
 
-#ifdef COMPAT_HAVE_ENCODING  /* see compat.h*/
 int msgpack_rb_encindex_utf8;
 int msgpack_rb_encindex_usascii;
 int msgpack_rb_encindex_ascii8bit;
-#endif
+
+ID s_uminus;
 
 #ifndef DISABLE_RMEM
 static msgpack_rmem_t s_rmem;
@@ -35,11 +35,11 @@ static msgpack_rmem_t s_rmem;
 
 void msgpack_buffer_static_init()
 {
-#ifdef COMPAT_HAVE_ENCODING
+    s_uminus = rb_intern("-@");
+
     msgpack_rb_encindex_utf8 = rb_utf8_encindex();
     msgpack_rb_encindex_usascii = rb_usascii_encindex();
     msgpack_rb_encindex_ascii8bit = rb_ascii8bit_encindex();
-#endif
 
 #ifndef DISABLE_RMEM
     msgpack_rmem_init(&s_rmem);
@@ -74,13 +74,13 @@ static void _msgpack_buffer_chunk_destroy(msgpack_buffer_chunk_t* c)
     if(c->mem != NULL) {
 #ifndef DISABLE_RMEM
         if(!msgpack_rmem_free(&s_rmem, c->mem)) {
-            free(c->mem);
+            xfree(c->mem);
         }
         /* no needs to update rmem_owner because chunks will not be
          * free()ed (left in free_list) and thus *rmem_owner is
          * always valid. */
 #else
-        free(c->mem);
+        xfree(c->mem);
 #endif
     }
     c->first = NULL;
@@ -95,7 +95,7 @@ void msgpack_buffer_destroy(msgpack_buffer_t* b)
     while(c != &b->tail) {
         msgpack_buffer_chunk_t* n = c->next;
         _msgpack_buffer_chunk_destroy(c);
-        free(c);
+        xfree(c);
         c = n;
     }
     _msgpack_buffer_chunk_destroy(c);
@@ -103,7 +103,7 @@ void msgpack_buffer_destroy(msgpack_buffer_t* b)
     c = b->free_list;
     while(c != NULL) {
         msgpack_buffer_chunk_t* n = c->next;
-        free(c);
+        xfree(c);
         c = n;
     }
 }
@@ -259,7 +259,7 @@ static inline msgpack_buffer_chunk_t* _msgpack_buffer_alloc_new_chunk(msgpack_bu
 {
     msgpack_buffer_chunk_t* reuse = b->free_list;
     if(reuse == NULL) {
-        return malloc(sizeof(msgpack_buffer_chunk_t));
+        return xmalloc(sizeof(msgpack_buffer_chunk_t));
     }
     b->free_list = b->free_list->next;
     return reuse;
@@ -308,9 +308,7 @@ static inline void _msgpack_buffer_add_new_chunk(msgpack_buffer_t* b)
 static inline void _msgpack_buffer_append_reference(msgpack_buffer_t* b, VALUE string)
 {
     VALUE mapped_string = rb_str_dup(string);
-#ifdef COMPAT_HAVE_ENCODING
     ENCODING_SET(mapped_string, msgpack_rb_encindex_ascii8bit);
-#endif
 
     _msgpack_buffer_add_new_chunk(b);
 
@@ -337,7 +335,6 @@ void _msgpack_buffer_append_long_string(msgpack_buffer_t* b, VALUE string)
 
     if(b->io != Qnil) {
         msgpack_buffer_flush(b);
-#ifdef COMPAT_HAVE_ENCODING
         if (ENCODING_GET(string) == msgpack_rb_encindex_ascii8bit) {
             rb_funcall(b->io, b->io_write_all_method, 1, string);
         } else if(!STR_DUP_LIKELY_DOES_COPY(string)) {
@@ -347,10 +344,6 @@ void _msgpack_buffer_append_long_string(msgpack_buffer_t* b, VALUE string)
         } else {
             msgpack_buffer_append(b, RSTRING_PTR(string), length);
         }
-#else
-        rb_funcall(b->io, b->io_write_all_method, 1, string);
-#endif
-
     } else if(!STR_DUP_LIKELY_DOES_COPY(string)) {
         _msgpack_buffer_append_reference(b, string);
 
@@ -403,7 +396,7 @@ static inline void* _msgpack_buffer_chunk_malloc(
 
     // TODO alignment?
     *allocated_size = required_size;
-    void* mem = malloc(required_size);
+    void* mem = xmalloc(required_size);
     c->mem = mem;
     return mem;
 }
@@ -421,7 +414,7 @@ static inline void* _msgpack_buffer_chunk_realloc(
         next_size *= 2;
     }
     *current_size = next_size;
-    mem = realloc(mem, next_size);
+    mem = xrealloc(mem, next_size);
 
     c->mem = mem;
     return mem;
