@@ -20,6 +20,7 @@
 
 #include <stdbool.h>
 #include "ruby.h"
+#include "ruby/encoding.h"
 
 #if defined(HAVE_RUBY_ST_H)
 #  include "ruby/st.h"  /* ruby hash on Ruby 1.9 */
@@ -29,15 +30,13 @@
 
 
 /*
- * COMPAT_HAVE_ENCODING
+ * ZALLOC_N (ruby 2.2 or later)
  */
-#ifdef HAVE_RUBY_ENCODING_H
-#  include "ruby/encoding.h"
-#  define COMPAT_HAVE_ENCODING
+#ifndef RB_ZALLOC_N
+#  define RB_ZALLOC_N(type,n) ((type*)ruby_xcalloc((size_t)(n),sizeof(type)))
 #endif
-
-#if defined(__MACRUBY__)  /* MacRuby */
-#  undef COMPAT_HAVE_ENCODING
+#ifndef ZALLOC_N
+#  define ZALLOC_N(type,n) RB_ZALLOC_N(type,n)
 #endif
 
 
@@ -110,5 +109,17 @@
 #endif
 
 
+/*
+ * RSTRUCT_GET
+ */
+#ifndef RSTRUCT_GET
+#  ifdef RSTRUCT_PTR  /* MRI <= 2.0.0 */
+#    define RSTRUCT_GET(st, idx)  (RSTRUCT_PTR(st)[idx])
+#  else /* Rubinius */
+#    define RSTRUCT_GET(st, idx)  (rb_struct_aref(st, INT2FIX(idx)))
+#  endif
+#endif
+
+
 #endif
 
