@@ -134,6 +134,41 @@ describe MessagePack::Packer do
     packer.to_s.should == "\x81"
   end
 
+  it 'write_bin_header 0' do
+    packer.write_bin_header(0)
+    packer.to_s.should == "\xC4\x00"
+  end
+
+  it 'write_bin_header 255' do
+    packer.write_bin_header(255)
+    packer.to_s.should == "\xC4\xFF"
+  end
+
+  it 'write_bin_header 256' do
+    packer.write_bin_header(256)
+    packer.to_s.should == "\xC5\x01\x00"
+  end
+
+  it 'write_bin_header 65535' do
+    packer.write_bin_header(65535)
+    packer.to_s.should == "\xC5\xFF\xFF"
+  end
+
+  it 'write_bin_header 65536' do
+    packer.write_bin_header(65536)
+    packer.to_s.should == "\xC6\x00\x01\x00\x00"
+  end
+
+  it 'write_bin_header 999999' do
+    packer.write_bin_header(999999)
+    packer.to_s.should == "\xC6\x00\x0F\x42\x3F"
+  end
+
+  it 'write_bin' do
+    packer.write_bin("hello")
+    packer.to_s.should == "\xC4\x05hello"
+  end
+
   describe '#write_float32' do
     tests = [
       ['small floats', 3.14, "\xCA\x40\x48\xF5\xC3"],
@@ -178,6 +213,29 @@ describe MessagePack::Packer do
     Array.new.to_msgpack.class.should == String
   end
 
+  it 'to_msgpack with packer equals to_msgpack' do
+    nil.to_msgpack(MessagePack::Packer.new).to_str.should == nil.to_msgpack
+    true.to_msgpack(MessagePack::Packer.new).to_str.should == true.to_msgpack
+    false.to_msgpack(MessagePack::Packer.new).to_str.should == false.to_msgpack
+    1.to_msgpack(MessagePack::Packer.new).to_str.should == 1.to_msgpack
+    1.0.to_msgpack(MessagePack::Packer.new).to_str.should == 1.0.to_msgpack
+    "".to_msgpack(MessagePack::Packer.new).to_str.should == "".to_msgpack
+    Hash.new.to_msgpack(MessagePack::Packer.new).to_str.should == Hash.new.to_msgpack
+    Array.new.to_msgpack(MessagePack::Packer.new).to_str.should == Array.new.to_msgpack
+  end
+
+  it 'raises type error on wrong type' do
+    packer = MessagePack::Packer.new
+    expect { packer.write_float "hello" }.to raise_error(TypeError)
+    expect { packer.write_string 1 }.to raise_error(TypeError)
+    expect { packer.write_bin 1 }.to raise_error(TypeError)
+    expect { packer.write_array "hello" }.to raise_error(TypeError)
+    expect { packer.write_hash "hello" }.to raise_error(TypeError)
+    expect { packer.write_symbol "hello" }.to raise_error(TypeError)
+    expect { packer.write_int "hello" }.to raise_error(TypeError)
+    expect { packer.write_extension "hello" }.to raise_error(TypeError)
+  end
+
   class CustomPack01
     def to_msgpack(pk=nil)
       return MessagePack.pack(self, pk) unless pk.class == MessagePack::Packer
@@ -350,6 +408,50 @@ describe MessagePack::Packer do
       end
     end
 
+    context 'when it and its super class has an ext type' do
+      before { stub_const('Value', Class.new) }
+      before do
+        Value.class_eval do
+          def to_msgpack_ext
+            'value_msgpacked'
+          end
+        end
+      end
+      before { packer.register_type(0x01, Value, :to_msgpack_ext) }
+
+      context "when it is a child class" do
+        before { stub_const('InheritedValue', Class.new(Value)) }
+        before do
+          InheritedValue.class_eval do
+            def to_msgpack_ext
+              'inherited_value_msgpacked'
+            end
+          end
+        end
+
+        before { packer.register_type(0x02, InheritedValue, :to_msgpack_ext) }
+        subject { packer.pack(InheritedValue.new).to_s }
+
+        it { is_expected.to eq "\xC7\x19\x02inherited_value_msgpacked" }
+      end
+
+      context "even when it is a child class" do
+        before { stub_const('InheritedValue', Class.new(Value)) }
+        before do
+          InheritedValue.class_eval do
+            def to_msgpack_ext
+              'inherited_value_msgpacked'
+            end
+          end
+        end
+
+        before { packer.register_type(0x02, InheritedValue, :to_msgpack_ext) }
+        subject { packer.pack(Value.new).to_s }
+
+        it { is_expected.to eq "\xC7\x0F\x01value_msgpacked" }
+      end
+    end
+
     context 'when it has no ext type but an included module has' do
       subject { packer.pack(Value.new).to_s }
 
